button = class:new()

function button:init(clickarea, drawable, drawarea, func, args)
	self.delete = false
	self.clickarea = clickarea
	self.drawable = drawable or true
	self.drawarea = drawarea or false
	self.func = func or false
	self.args = args or false
	self.active = true
	
	self.highlighted = false
	
	print("New button made as \"" .. self.clickarea[1] .. "\", drawn as \"" .. self.drawarea[1] .. "\"")
	--[[
		Click areas:
		
		Rectangle: {"rectangle", x, y, width, height}
		Circle: {"circle", x, y, radius, "center" or "corner"}
		Image: {"image", x, y, imagedata}
		
		
		Draw areas:
		Rectangle: {"rectangle", x, y, width, height, color, outlinecolor, outlinesize}
		Circle: {"circle", x, y, radius, "center" or "corner", color, outlinecolor, outlinesize}
		Image: {"image", x, y, image, color}
	]]
end

function button:update(dt)
	local x, y = love.mouse.getX(), love.mouse.getY()
	local clc = self.clickarea
	self.highlighted = false
	if type(clc) == "table" and self.active then
		if clc[1] == "rectangle" then
			if x >= clc[2] and x < clc[2]+clc[4] and y >= clc[3] and y < clc[3]+clc[5] then
				self.highlighted = true
			end
		elseif self.clickarea[2] == "circle" then
			if clc[5] == "center" then
				if getDistance(x, y, clc[2], clc[3]) <= clc[4] then
					self.highlighted = true
				end
			elseif clc[5] == "corner" or not clc[5] then
				if getDistance(x, y, clc[2]+clc[4], clc[3]+clc[4]) <= clc[4] then
					self.highlighted = true
				end
			end
		elseif self.clickarea[3] == "image" then
			if x >= clc[2] and x < clc[2]+clc[4]:getWidth() and y >= clc[3] and y < clc[4]:getHeight() then
				local r, g, b, a = clc[4]:getPixel(x-clc[2], y-clc[3])
				if a > 128 then
					self.highlighted = true
				end
			end
		end
	end
	return self.delete
end

function button:draw()
	if self.drawable then
		if self.drawarea then
			local da = self.drawarea
			if da[1] == "rectangle" then
				love.graphics.setColor(unpack(da[6]))
				love.graphics.rectangle("fill", da[2], da[3], da[4], da[5])
				if da[7] then
					love.graphics.setColor(unpack(da[7]))
					if da[8] then
						love.graphics.setLine(da[8])
					end
					love.graphics.rectangle("line", da[2], da[3], da[4], da[5])
					love.graphics.setLine(1)
				end
			elseif da[1] == "circle" then
				love.graphics.setColor(unpack(da[5]))
				if da[5] == "center" then
					love.graphics.circle("fill", da[2], da[3], da[4], 32)
				else
					love.graphics.circle("fill", da[2]+da[4], da[3]+da[4], da[4], 32)
				end
				if da[7] then
					love.graphics.setColor(unpack(da[7]))
					if da[8] then
						love.graphics.setLine(da[8])
					end
					if da[5] == "center" then
						love.graphics.circle("line", da[2], da[3], da[4], 32)
					else
						love.graphics.circle("line", da[2]+da[4], da[3]+da[4], da[4], 32)
					end
					love.graphics.setLine(1)
				end
			elseif da[1] == "image" then
				if da[5] then
					love.graphics.setColor(unpack(da[5]))
				end
				love.graphics.draw(da[4], da[2], da[3])
			end
		end
	end
end

function button:keypressed(key, unicode)
	
end

function button:keyreleased(key, unicode)
	
end

function button:mousepressed(x, y, button)
	local clc = self.clickarea
	if type(clc) == "table" and self.active then
		if clc[1] == "rectangle" then
			if x >= clc[2] and x < clc[2]+clc[4] and y >= clc[3] and y < clc[3]+clc[5] then
				self:exec()
			end
		elseif clc[1] == "circle" then
			if clc[5] == "center" then
				if getDistance(x, y, clc[2], clc[3]) <= clc[4] then
					self:exec()
				end
			elseif clc[5] == "corner" or not clc[5] then
				if getDistance(x, y, clc[2]+clc[4], clc[3]+clc[4]) <= clc[4] then
					self:exec()
				end
			end
		elseif clc[1] == "image" then
			if x >= clc[2] and x < clc[2]+clc[4]:getWidth() and y >= clc[3] and y < clc[3]+clc[4]:getHeight() then
				print("Click on area")
				local r, g, b, a = clc[4]:getPixel(x-clc[2], y-clc[3])
				if a > 128 then
					self:exec()
					print("Executed")
				end
			end
		end
	end
end

function button:mousereleased(x, y, button)
	
end

function button:exec()
	if self.func then
		if self.args then
			if type(self.args) == "table" then
				self.func(unpack(self.args))
			else
				self.func(self.args)
			end
		else
			self.func()
		end
		frozenclick = true
	end
end