window = class:new()

windowside = 3
windowtop = 20
windowminw = 100
windowminh = 50

function window:init(x, y, width, height, wtype, title, data, popup)
	self.delete = false
	
	self.isfocused = true
	focusedwindow = self
	self.x = x
	self.y = y
	self.width = width+2*windowside
	self.height = height+windowside+windowtop
	self.type = wtype or "text" -- text, image and music
	self.title = title or "Untitled"
	self.data = data or false
	self.popup = popup or false
	self.minw = windowminw
	self.minh = windowminh
	
	self.breakline = true
	
	if self.type == "music" or self.type == "image" then
		self.minw = self.width
		self.minh = self.height
	end
	
	if self.type == "music" and self.data then
		self.volume = 1
		local posx = self.x+windowside
		local posy = self.y+windowtop
		
		local playimgdata = love.image.newImageData(GraphFolder .. "/music/play.png")
		local clickarea = {"image", posx, posy, playimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.play}
		table.insert(buttons, button:new(clickarea, false, drawarea, playmusic, {self}))
		self.playbutton = #buttons
		buttons[self.playbutton].active = false
		local pauseimgdata = love.image.newImageData(GraphFolder .. "/music/pause.png")
		local clickarea = {"image", posx, posy, pauseimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.pause}
		table.insert(buttons, button:new(clickarea, true, drawarea, pausemusic, {self}))
		self.pausebutton = #buttons
		local forwardimgdata = love.image.newImageData(GraphFolder .. "/music/forward.png")
		local clickarea = {"image", posx, posy, forwardimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.forward}
		table.insert(buttons, button:new(clickarea, true, drawarea))
		self.forwardbutton = #buttons
		local backwardimgdata = love.image.newImageData(GraphFolder .. "/music/backward.png")
		local clickarea = {"image", posx, posy, backwardimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.backward}
		table.insert(buttons, button:new(clickarea, true, drawarea))
		self.backwardbutton = #buttons
		local stopimgdata = love.image.newImageData(GraphFolder .. "/music/stop.png")
		local clickarea = {"image", posx, posy, stopimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.stop}
		table.insert(buttons, button:new(clickarea, true, drawarea, stopmusic, {self}))
		self.stopbutton = #buttons
		local vol_increaseimgdata = love.image.newImageData(GraphFolder .. "/music/vol_increase.png")
		local clickarea = {"image", posx, posy, vol_increaseimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.vol_increase}
		table.insert(buttons, button:new(clickarea, true, drawarea, setvolume, {self, "increase"}))
		self.vol_increasebutton = #buttons
		local vol_decreaseimgdata = love.image.newImageData(GraphFolder .. "/music/vol_decrease.png")
		local clickarea = {"image", posx, posy, vol_decreaseimgdata}
		local drawarea = {"image", posx, posy, musicbuttons.vol_decrease}
		table.insert(buttons, button:new(clickarea, true, drawarea, setvolume, {self, "decrease"}))
		self.vol_decreasebutton = #buttons
	end
	
	self.scroll = 0
	
	self.closebutton = {	x = self.x+self.width-windowside*2-10,
							y = self.y+windowside*2,
							width = 10, height = 10}
	self.windowbutton = {	x = self.x+self.width-windowside*3-20,
							y = self.y+windowside*2,
							width = 10, height = 10}
	self.minimizebutton = {	x = self.x+self.width-windowside*4-30,
							y = self.y+windowside*2,
							width = 10, height = 10}
	
	if self.type == "music" and self.data then
		love.audio.play(self.data)
	end
	
	self.resizing = false
	self.moving = false
	self.movingoffsets = {x = 0, y = 0}
	self.resizingside = 0
	self.resizingoffsets = {x = 0, y = 0}
	
	self.lastdata = {x = self.x, y = self.y, width = self.width, height = self.height}
	
	self.resizehighlight = false
	self.imagezoom = 1
	
	self.maximized = false
	
	--[[
	  NUMBER |   POSITION	 |	 X1 ; X2	 |	 Y1 ; Y2
	
	  1:	   top left			 -3 ; 3			 -3 ; 3
	  2:	   top				  3 ; w-3		 -3 ; 3
	  3:	   top right		w-3 ; w+3		 -3 ; 3
	  4:	   left				 -3 ; 3			  3 ; h-3
	  5:	   right			w-3 ; w+3		  3 ; h-3
	  6:	   bottom left		 -3 ; 3			h-3 ; h+3
	  7:	   bottom			  3 ; w-3		h-3 ; h+3
	  8:	   bottom right		w-3 ; w+3		h-3 ; h+3
	]]
end

function playmusic(t)
	t.data:play()
	print("Played")
	buttons[t.pausebutton].active = true
	buttons[t.playbutton].active = false
	buttons[t.pausebutton].drawable = true
	buttons[t.playbutton].drawable = false
end

function pausemusic(t)
	t.data:pause()
	print("Paused")
	buttons[t.pausebutton].active = false
	buttons[t.playbutton].active = true
	buttons[t.pausebutton].drawable = false
	buttons[t.playbutton].drawable = true
end

function stopmusic(t)
	pausemusic(t) --So I get the play button again
	t.data:stop()
	print("Stopped")
end

function setvolume(t, value)
	if value == "increase" then
		if t.volume < 1 then
			t.volume = t.volume + .25
		end
	elseif value == "decrease" then
		if t.volume > 0 then
			t.volume = t.volume - .25
		end
	elseif value >= 0 and value <= 1 then
		t.volume = value
	end
	
	t.data:setVolume(t.volume)
end

function window:movebuttons()
	local posx = self.x+windowside
	local posy = self.y+windowtop
	
	buttons[self.playbutton].clickarea[2] = posx
	buttons[self.playbutton].drawarea[2] = posx
	buttons[self.playbutton].clickarea[3] = posy
	buttons[self.playbutton].drawarea[3] = posy
	buttons[self.pausebutton].clickarea[2] = posx
	buttons[self.pausebutton].drawarea[2] = posx
	buttons[self.pausebutton].clickarea[3] = posy
	buttons[self.pausebutton].drawarea[3] = posy
	buttons[self.forwardbutton].clickarea[2] = posx
	buttons[self.forwardbutton].drawarea[2] = posx
	buttons[self.forwardbutton].clickarea[3] = posy
	buttons[self.forwardbutton].drawarea[3] = posy
	buttons[self.backwardbutton].clickarea[2] = posx
	buttons[self.backwardbutton].drawarea[2] = posx
	buttons[self.backwardbutton].clickarea[3] = posy
	buttons[self.backwardbutton].drawarea[3] = posy
	buttons[self.stopbutton].clickarea[2] = posx
	buttons[self.stopbutton].drawarea[2] = posx
	buttons[self.stopbutton].clickarea[3] = posy
	buttons[self.stopbutton].drawarea[3] = posy
	buttons[self.vol_increasebutton].clickarea[2] = posx
	buttons[self.vol_increasebutton].drawarea[2] = posx
	buttons[self.vol_increasebutton].clickarea[3] = posy
	buttons[self.vol_increasebutton].drawarea[3] = posy
	buttons[self.vol_decreasebutton].clickarea[2] = posx
	buttons[self.vol_decreasebutton].drawarea[2] = posx
	buttons[self.vol_decreasebutton].clickarea[3] = posy
	buttons[self.vol_decreasebutton].drawarea[3] = posy
end

function window:update(dt)
	if self.isfocused and not self.moving and not self.maximized then --Get new mouse texture
		mouseimg = mouse.default[mousecolor]
		local x = love.mouse.getX()
		local y = love.mouse.getY()
		-- RESIZE 1: Top-Left
		if x >= self.x-windowside and x <= self.x+windowside and y >= self.y-windowside and y <= self.y+windowside then
			mouseimg = mouse.dia1[mousecolor]
			self.resizehighlight = true
		-- RESIZE 2: Top
		elseif x >= self.x+windowside and x <= self.x+self.width-windowside and y >= self.y-windowside and y <= self.y+windowside then
			mouseimg = mouse.ver[mousecolor]
			self.resizehighlight = true
		-- RESIZE 3: Top-Right
		elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y-windowside and y <= self.y+windowside then
			mouseimg = mouse.dia2[mousecolor]
			self.resizehighlight = true
		-- RESIZE 4: Left
		elseif x >= self.x-windowside and x <= self.x+windowside and y >= self.y+windowside and y <= self.y+self.height-windowside then
			mouseimg = mouse.hor[mousecolor]
			self.resizehighlight = true
		-- RESIZE 5: Right
		elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y+windowside and y <= self.y+self.height-windowside then
			mouseimg = mouse.hor[mousecolor]
			self.resizehighlight = true
		-- RESIZE 6: Bottom-Left
		elseif x >= self.x-windowside and x <= self.x+windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
			mouseimg = mouse.dia1[mousecolor]
			self.resizehighlight = true
		-- RESIZE 7: Bottom
		elseif x >= self.x+windowside and x <= self.x+self.width-windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
			mouseimg = mouse.ver[mousecolor]
			self.resizehighlight = true
		-- RESIZE 8: Bottom-Right
		elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
			mouseimg = mouse.dia1[mousecolor]
			self.resizehighlight = true
		else
			self.resizehighlight = false
		end
	else
		self.resizehighlight = false
	end
	
	if self.moving then
		self.x = love.mouse.getX()-self.movingoffsets.x
		self.y = love.mouse.getY()-self.movingoffsets.y
		if self.type == "music" and self.data then
			self:movebuttons()
		end
	elseif self.resizing then
		if self.resizingside == 1 then
			self.x = love.mouse.getX()-self.resizingoffsets.x
			self.y = love.mouse.getY()-self.resizingoffsets.y
			self.width = self.lastdata.x-self.x+self.lastdata.width
			self.height = self.lastdata.y-self.y+self.lastdata.height
		elseif self.resizingside == 2 then
			self.y = love.mouse.getY()-self.resizingoffsets.y
			self.height = self.lastdata.y-self.y+self.lastdata.height
		elseif self.resizingside == 3 then
			self.y = love.mouse.getY()-self.resizingoffsets.y
			self.width = love.mouse.getX()-self.resizingoffsets.x-self.lastdata.x+self.lastdata.width
			self.height = self.lastdata.y-self.y+self.lastdata.height
		elseif self.resizingside == 4 then
			self.x = love.mouse.getX()-self.resizingoffsets.x
			self.width = self.lastdata.x-self.x+self.lastdata.width
		elseif self.resizingside == 5 then
			self.width = love.mouse.getX()-self.resizingoffsets.x-self.lastdata.x+self.lastdata.width
		elseif self.resizingside == 6 then
			self.x = love.mouse.getX()-self.resizingoffsets.x
			self.width = self.lastdata.x-self.x+self.lastdata.width
			self.height = love.mouse.getY()-self.resizingoffsets.y-self.lastdata.y+self.lastdata.height
		elseif self.resizingside == 7 then
			self.height = love.mouse.getY()-self.resizingoffsets.y-self.lastdata.y+self.lastdata.height
		elseif self.resizingside == 8 then
			self.width = love.mouse.getX()-self.resizingoffsets.x-self.lastdata.x+self.lastdata.width
			self.height = love.mouse.getY()-self.resizingoffsets.y-self.lastdata.y+self.lastdata.height
		end
		
		if self.width < self.minw then
			self.width = self.minw
		end
		
		if self.height < self.minh then
			self.height = self.minh
		end
		
		if self.type == "music" and self.data then
			self:movebuttons()
		end
	end
	
	if self.moving or self.resizing then
		self.closebutton = {	x = self.x+self.width-windowside*2-10,
								y = self.y+windowside*2,
								width = 10, height = 10}
		self.windowbutton = {	x = self.x+self.width-windowside*3-20,
								y = self.y+windowside*2,
								width = 10, height = 10}
		self.minimizebutton = {	x = self.x+self.width-windowside*4-30,
								y = self.y+windowside*2,
								width = 10, height = 10}
	end
end

function window:draw()
	if self.isfocused then
		love.graphics.setColor(unpack(windowfocusedback))
		love.graphics.rectangle("fill", self.x+windowside, self.y+windowtop, self.width-2*windowside, self.height-windowside-windowtop)
		love.graphics.setColor(unpack(windowfocusedcolor))
	else
		love.graphics.setColor(unpack(windowunfocusedback))
		love.graphics.rectangle("fill", self.x+windowside, self.y+windowtop, self.width-2*windowside, self.height-windowside-windowtop)
		love.graphics.setColor(unpack(windowunfocusedcolor))
	end
	
	love.graphics.rectangle("fill", self.x, self.y, self.width, windowtop) --Top bar
	love.graphics.rectangle("fill", self.x, self.y+windowtop, windowside, self.height-windowtop-windowside) --Left bar
	love.graphics.rectangle("fill", self.x+self.width-windowside, self.y+windowtop, windowside, self.height-windowtop-windowside) --Right bar
	love.graphics.rectangle("fill", self.x, self.y+self.height-windowside, self.width, windowside) --Right bar
	
	if self.isfocused then
		love.graphics.setColor(unpack(windowfocusedtitle))
	else
		love.graphics.setColor(unpack(windowunfocusedtitle))
	end
	
	local newtxt = self.title
	if font:getWidth(newtxt) > self.width-6*windowside-30 then
		newtxt = self.title .. "..."
		local can = true
		while font:getWidth(newtxt) > self.width-6*windowside-30 and can do
			newtxt = string.sub(newtxt, 1, -5) .. "..."
			if string.len(newtxt) == 6 then
				can = false
			end
		end
	end
	love.graphics.print(newtxt, self.x+windowside, self.y+windowside)
	love.graphics.setScissor(self.x+windowside, self.y+windowtop, self.width-2*windowside,self.height-windowside-windowtop)
	if self.type == "text" then
		if self.data then
			love.graphics.setFont(textfont)
			if self.isfocused then
				love.graphics.setColor(unpack(windowfocusedtext))
			else
				love.graphics.setColor(unpack(windowunfocusedtext))
			end
			if self.breakline then
				love.graphics.printf(self.data, self.x+2*windowside, self.y+windowtop+windowside-self.scroll, self.width-4*windowside)
			else
				local txt = self.data
				love.graphics.printf(self.data, self.x+2*windowside, self.y+windowtop+windowside-self.scroll, string.len(self.data))
			end
			love.graphics.setFont(font)
		end
	elseif self.type == "image" then
		if self.data then
			love.graphics.setColor(255, 255, 255, 255)
			local posx = self.x+windowside+(self.width-windowside*2)/2-(self.minw-windowside*2)/2
			posx = posx + (self.data:getWidth() - self.data:getWidth()*self.imagezoom)/2
			local posy = self.y+windowtop+(self.height-windowside-windowtop)/2-(self.minh-windowside-windowtop)/2
			posy = posy + (self.data:getHeight() - self.data:getHeight()*self.imagezoom)/2
			love.graphics.draw(self.data, posx, posy, 0, self.imagezoom, self.imagezoom)
		end
	elseif self.type == "music" then
		if self.data then
			love.graphics.draw(musicbuttons.vol_back, self.x+windowside, self.y+windowtop)
			local posy = (-self.volume+1)*16*4
			love.graphics.draw(musicbuttons.vol_slider, self.x+windowside, self.y+windowtop+posy)
		end
	end
	
	love.graphics.setScissor()
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(closebutton, self.closebutton.x, self.closebutton.y, 0, 1, 1)
	if self.maximized then
		love.graphics.draw(windowbutton, self.windowbutton.x, self.windowbutton.y, 0, 1, 1)
	else
		love.graphics.draw(maximizebutton, self.windowbutton.x, self.windowbutton.y, 0, 1, 1)
	end
	love.graphics.draw(minimizebutton, self.minimizebutton.x, self.minimizebutton.y, 0, 1, 1)
end

function window:keypressed(key, unicode)
	if key == "b" then
		self.breakline = not self.breakline
	end
	
	if key == "audioplay" then
		if self.type == "music" and self.data then
			if self.data:isPaused() then
				playmusic(self)
			else
				pausemusic(self)
			end
		end
	end
end

function window:keyreleased(key, unicode)
	
end

function window:mousepressed(x, y, button)
	if button == "l" then
		if x >= self.x and x <= self.x+self.width and y >= self.y and y <= self.y+self.height and not self.isfocused then
			self.isfocused = true
			focusedwindow = self
			print("Focused window")
		elseif self.isfocused then
			self.isfocused = true
			if x >= self.closebutton.x and x <= self.closebutton.x+self.closebutton.width and y >= self.closebutton.y and y <= self.closebutton.y+self.closebutton.height then --Close
				if self.popup then
					-- TO DO
				else
					self:close()
					if self.isfocused then
						focusedwindow = nil
					end
				end
			elseif x >= self.windowbutton.x and x <= self.windowbutton.x+self.windowbutton.width and y >= self.windowbutton.y and y <= self.windowbutton.y+self.windowbutton.height then --Set maximized
				self.maximized = not self.maximized
				if self.maximized then
					self.lastdata = {x = self.x, y = self.y, width = self.width, height = self.height}
					self.x = 0
					self.y = 0
					self.width = screenwidth
					self.height = screenheight
				else
					self.x = self.lastdata.x
					self.y = self.lastdata.y
					self.width = self.lastdata.width
					self.height = self.lastdata.height
				end
				self.closebutton = {	x = self.x+self.width-windowside*2-10,
										y = self.y+windowside*2,
										width = 10, height = 10}
				self.windowbutton = {	x = self.x+self.width-windowside*3-20,
										y = self.y+windowside*2,
										width = 10, height = 10}
				self.minimizebutton = {	x = self.x+self.width-windowside*4-30,
										y = self.y+windowside*2,
										width = 10, height = 10}
			elseif x >= self.minimizebutton.x and x <= self.minimizebutton.x+self.minimizebutton.width and y >= self.minimizebutton.y and y <= self.minimizebutton.y+self.minimizebutton.height then --Minimize
				-- TO DO
			elseif not self.maximized then
				self.lastdata = {x = self.x, y = self.y, width = self.width, height = self.height}
				if x > self.x+windowside and x < self.x+self.width-windowside and y > self.y+windowside and y < self.y+windowtop-windowside then --MOVING
					self.moving = true
					self.movingoffsets = {x = x-self.x, y = y-self.y}
					mouseimg = mouse.move[mousecolor]
				-- RESIZE 1: Top-Left
				elseif x >= self.x-windowside and x <= self.x+windowside and y >= self.y-windowside and y <= self.y+windowside then
					self.resizing = true
					self.resizingside = 1
					mouseimg = mouse.dia1[mousecolor]
				-- RESIZE 2: Top
				elseif x >= self.x+windowside and x <= self.x+self.width-windowside and y >= self.y-windowside and y <= self.y+windowside then
					self.resizing = true
					self.resizingside = 2
					mouseimg = mouse.ver[mousecolor]
				-- RESIZE 3: Top-Right
				elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y-windowside and y <= self.y+windowside then
					self.resizing = true
					self.resizingside = 3
					mouseimg = mouse.dia2[mousecolor]
				-- RESIZE 4: Left
				elseif x >= self.x-windowside and x <= self.x+windowside and y >= self.y+windowside and y <= self.y+self.height-windowside then
					self.resizing = true
					self.resizingside = 4
					mouseimg = mouse.hor[mousecolor]
				-- RESIZE 5: Right
				elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y+windowside and y <= self.y+self.height-windowside then
					self.resizing = true
					self.resizingside = 5
					mouseimg = mouse.hor[mousecolor]
				-- RESIZE 6: Bottom-Left
				elseif x >= self.x-windowside and x <= self.x+windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
					self.resizing = true
					self.resizingside = 6
					mouseimg = mouse.dia1[mousecolor]
				-- RESIZE 7: Bottom
				elseif x >= self.x+windowside and x <= self.x+self.width-windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
					self.resizing = true
					self.resizingside = 7
					mouseimg = mouse.ver[mousecolor]
				-- RESIZE 8: Bottom-Right
				elseif x >= self.x+self.width-windowside and x <= self.x+self.width+windowside and y >= self.y+self.height-windowside and y <= self.y+self.height+windowside then
					self.resizing = true
					self.resizingside = 8
					mouseimg = mouse.dia1[mousecolor]
				end
				
				if self.resizing then
					self.resizingoffsets = {x = x-self.x, y = y-self.y}
				elseif x < self.x or x > self.x+self.width or y < self.y or y > self.y+self.height and not self.resizehighlight then
					--Out of focus!
					if self.isfocused then
						focusedwindow = nil
					end
					self.isfocused = false
				end
			end
		end
	elseif button == "wu" then
		if self.type == "text" then
			if self.scroll > 0 then
				self.scroll = self.scroll - scrollfactor
			end
		elseif self.type == "image" then
			self.imagezoom = self.imagezoom + .25
		end
	elseif button == "wd" then
		if self.type == "text" then
			self.scroll = self.scroll + scrollfactor
		elseif self.type == "image" then
			if self.imagezoom > 1 then
				self.imagezoom = self.imagezoom - .25
			end
		end
	end
end

function window:mousereleased(x, y, button)
	self.moving = false
	self.resizing = false
end

function window:minimize()

end

function window:maximize()

end

function window:restore()

end

function window:close()
	if not self.popup then
		self:forceclose()
	else
		table.insert(windows, self.popup)
	end
end

function window:forceclose()
	if self.type == "music" and self.data then --Stop it!
		self.data:stop()
		buttons[self.playbutton] = nil
		buttons[self.pausebutton] = nil
		buttons[self.forwardbutton] = nil
		buttons[self.backwardbutton] = nil
		buttons[self.stopbutton] = nil
		buttons[self.vol_increasebutton] = nil
		buttons[self.vol_decreasebutton] = nil
	end
	self.delete = true
	frozenclick = true
end