--[[
       ######################################################################################
     ##                                                                                      ##
    #    HUGOBDESIGNER'S FILE MANAGER       LICENSE:                                           #
    #                by HugoBDesigner       http://creativecommons.org/licenses/by-sa/3.0/     #
    #                                                                                          #
    #        Hello, and welcome to my                                                        ##
    #    little file manager. This is       #################################################
    #    an experiment I've wanted to      #
    #    do for quite a long time. It       #################################################
    #    is just a small test for me,                                                        ##
    #    to test my skills. Any idea,       CONTACT PAGE:                                      #
    #    comment or suggestion, visit       http://hugobdesigner.blogspot.com/p/contact.html   #
    #    my blog's contact page.                                                               #
     ##                                                                                      ##
       ######################################################################################
]]

love.filesystem.enumerate = love.filesystem.enumerate or love.filesystem.getDirectoryItems
love.graphics.drawq = love.graphics.drawq or love.graphics.draw
love.graphics.setCaption = love.graphics.setCaption or love.window.setTitle
love.graphics.setIcon = love.graphics.setIcon or love.window.setIcon
love.filesystem.mkdir = love.filesystem.mkdir or love.filesystem.createDirectory
love.graphics.setDefaultImageFilter = love.graphics.setDefaultImageFilter or love.graphics.setDefaultFilter
love.graphics.setLine = love.graphics.setLine or love.graphics.setLineWidth
rprint = print

function love.load()
	-- First of all, get if users (like me) want to open the console
	if love.filesystem.exists("console.txt") then
		local txt = love.filesystem.read("console.txt")
		if txt == "true" then
			if love._openConsole then
				love._openConsole()
			end
		end
	end
	
	-- Set up the Image Filter
	love.graphics.setDefaultImageFilter("nearest", "nearest")
	
	--Then, set things for in-game "console"
	consoleenabled = false
	consoletexts = {}
	totalcoms = 10
	
	-- Setting up shortcuts
	GraphFolder = "HBD_fileManager/graphics"
	
	-- Listing and organizing icons
	images = {}
	images.folders = {}
	images.folders.empty = love.graphics.newImage(GraphFolder .. "/folder_empty.png")
	images.folders.doc = {}
	images.folders.doc.empty = love.graphics.newImage(GraphFolder .. "/folder_doc_empty.png")
	images.folders.doc.image = love.graphics.newImage(GraphFolder .. "/folder_doc_image.png")
	images.folders.doc.multi = love.graphics.newImage(GraphFolder .. "/folder_doc_multi.png")
	images.folders.doc.music = love.graphics.newImage(GraphFolder .. "/folder_doc_music.png")
	images.folders.doc.text = love.graphics.newImage(GraphFolder .. "/folder_doc_text.png")
	images.folders.doc.video = love.graphics.newImage(GraphFolder .. "/folder_doc_video.png")
	images.docs = {}
	images.docs.empty = love.graphics.newImage(GraphFolder .. "/doc_empty.png")
	images.docs.image = love.graphics.newImage(GraphFolder .. "/doc_image.png")
	images.docs.multi = love.graphics.newImage(GraphFolder .. "/doc_multi.png")
	images.docs.music = love.graphics.newImage(GraphFolder .. "/doc_music.png")
	images.docs.text = love.graphics.newImage(GraphFolder .. "/doc_text.png")
	images.docs.video = love.graphics.newImage(GraphFolder .. "/doc_video.png")
	
	-- Identify the known file extensions
	ext = {}
	ext.text = {".txt", ".lua"}
	ext.image = {".png", ".png", ".tga", ".gif", ".bmp"}
	ext.video = {".avi", ".mp4", ".flv", ".mpg"}
	ext.music = {".mp3", ".wav", ".ogg"}
	
	-- Getting all the files
	files = recursiveEnumerate("")
	filespos = {}
	filesimages = {}
	totalfiles = 0
	highlighted = {}
	hltimer = 0
	for i, v in pairs(files) do
		totalfiles = totalfiles + 1
		filespos[totalfiles] = i
		if type(v) == "string" then
			print("Files["..i.."] = " .. tostring(v))
		else
			print("Files["..i.."] amount of files = " .. tostring(#v))
		end
		local posy = math.ceil(totalfiles/10)
		local posx = math.mod(totalfiles+10, 10)
		local img = images.docs.empty
		if posx == 0 then posx = 10 end
		local txt = v
		if type(v) == "table" then -- It's a folder
			local istext, isimage, isvideo, ismusic, isfile = 0, 0, 0, 0, false
			img = images.folders.empty
			for j, w in pairs(v) do
				if type(w) == "string" then -- It has files
					isfile = true
					local extension = string.sub(w, -4, -1)
					if table.contains(ext.text, extension) then
						istext = 1
						img = images.folders.doc.text
					elseif table.contains(ext.image, extension) then
						isimage = 1
						img = images.folders.doc.image
					elseif table.contains(ext.video, extension) then
						isvideo = 1
						img = images.folders.doc.video
					elseif table.contains(ext.music, extension) then
						ismusic = 1
						img = images.folders.doc.music
					end
				end
			end
			if isfile then
				if istext+isimage+isvideo+ismusic > 1 then
					img = images.folders.doc.multi
				end
			end
			txt = i
		elseif love.filesystem.isFile(v) then
			local istext, isimage, isvideo, ismusic = 0, 0, 0, 0
			local extension = string.sub(v, -4, -1)
			if table.contains(ext.text, extension) then
				istext = 1
				img = images.docs.text
			elseif table.contains(ext.image, extension) then
				isimage = 1
				img = images.docs.image
			elseif table.contains(ext.video, extension) then
				isvideo = 1
				img = images.docs.video
			elseif table.contains(ext.music, extension) then
				ismusic = 1
				img = images.docs.music
			end
			if istext+isimage+isvideo+ismusic > 1 then
				img = images.docs.multi
			end
			txt = string.sub(v, 1, -5)
		end
		filesimages[i] = {}
		filesimages[i].image = img
		filesimages[i].x = posx
		filesimages[i].y = posy
		filesimages[i].text = txt
	end
	print("Total amount of files = " .. totalfiles)
	
	-- Make up the grid for files
	
	gridoffsets = {x = 20, y = 50}
	griddistances = {x = 64, y = 80}
	grid = {}
	for y = 1, math.ceil(totalfiles/10) do
		grid[y] = {}
		for x = 1, 10 do
			grid[y][x] = {}
			grid[y][x].x = gridoffsets.x + (x-1)*griddistances.x
			grid[y][x].y = gridoffsets.y + (y-1)*griddistances.y
		end
	end
	
	-- Setting up the best LÖVE environment to work on
	screenwidth = 1024
	screenheight = 768
	fullscreen = true
	if love._version == "0.8.0" then
		love.graphics.setMode(screenwidth, screenheight, fullscreen)
	else
		love.window.setMode(screenwidth, screenheight, {fullscreen = fullscreen})
	end
	
	--Colors and images
	love.graphics.setBackgroundColor(192, 255, 128, 255) -- Cute green
	
	windowfocusedcolor = {155, 204, 104, 255} -- Dark "cute" green
	windowunfocusedcolor = {155, 155, 155, 155} -- Dark grey
	windowfocusedtitle = {255, 255, 255, 255} -- White
	windowunfocusedtitle = {105, 105, 105, 255} -- Dark grey
	windowfocusedtext = {0, 0, 0, 255} -- Black
	windowunfocusedtext = {55, 55, 55, 255} -- Dark, dark grey
	windowfocusedback = {218, 255, 178, 255} -- Light "cute" green
	windowunfocusedback = {205, 205, 205, 155} -- Light gret
	
	mousecolor = "white"
	mouse = {}
	mouse.default = {}
	mouse.move = {}
	mouse.hor = {}
	mouse.ver = {}
	mouse.dia1 = {}
	mouse.dia2 = {}
	
	mouse.default["white"] = 	love.graphics.newImage(GraphFolder .. "/mouse_white/default.png")
	mouse.move["white"] = 		love.graphics.newImage(GraphFolder .. "/mouse_white/move.png")
	mouse.hor["white"] = 	love.graphics.newImage(GraphFolder .. "/mouse_white/res_hor.png")
	mouse.ver["white"] = 	love.graphics.newImage(GraphFolder .. "/mouse_white/res_ver.png")
	mouse.dia1["white"] = 	love.graphics.newImage(GraphFolder .. "/mouse_white/res_dia1.png")
	mouse.dia2["white"] = 	love.graphics.newImage(GraphFolder .. "/mouse_white/res_dia2.png")
	
	mouse.default["black"] = 	love.graphics.newImage(GraphFolder .. "/mouse_black/default.png")
	mouse.move["black"] = 		love.graphics.newImage(GraphFolder .. "/mouse_black/move.png")
	mouse.hor["black"] = 	love.graphics.newImage(GraphFolder .. "/mouse_black/res_hor.png")
	mouse.ver["black"] = 	love.graphics.newImage(GraphFolder .. "/mouse_black/res_ver.png")
	mouse.dia1["black"] = 	love.graphics.newImage(GraphFolder .. "/mouse_black/res_dia1.png")
	mouse.dia2["black"] = 	love.graphics.newImage(GraphFolder .. "/mouse_black/res_dia2.png")
	
	closebutton = love.graphics.newImage(GraphFolder .. "/windows_buttons/close.png")
	windowbutton = love.graphics.newImage(GraphFolder .. "/windows_buttons/window.png")
	maximizebutton = love.graphics.newImage(GraphFolder .. "/windows_buttons/maximize.png")
	minimizebutton = love.graphics.newImage(GraphFolder .. "/windows_buttons/minimize.png")
	
	musicbuttons = {}
	musicbuttons.play = love.graphics.newImage(GraphFolder .. "/music/play.png")
	musicbuttons.pause = love.graphics.newImage(GraphFolder .. "/music/pause.png")
	musicbuttons.forward = love.graphics.newImage(GraphFolder .. "/music/forward.png")
	musicbuttons.backward = love.graphics.newImage(GraphFolder .. "/music/backward.png")
	musicbuttons.stop = love.graphics.newImage(GraphFolder .. "/music/stop.png")
	musicbuttons.vol_increase = love.graphics.newImage(GraphFolder .. "/music/vol_increase.png")
	musicbuttons.vol_decrease = love.graphics.newImage(GraphFolder .. "/music/vol_decrease.png")
	musicbuttons.vol_slider = love.graphics.newImage(GraphFolder .. "/music/vol_slider.png")
	musicbuttons.vol_back = love.graphics.newImage(GraphFolder .. "/music/vol_back.png")
	
	-- Variables
	mouseimg = mouse.default[mousecolor]
	love.mouse.setVisible(false)
	
	font = love.graphics.newFont("HBD_fileManager/font.ttf")
	textfont = love.graphics.newFont("HBD_fileManager/textfont.ttf")
	love.graphics.setFont(font)
	scroll = 0
	iconScale = 2
	scrollfactor = 8
	clicktimer = .5
	frozenclick = false
	
	filesscale = 2
	
	-- Extra requirements
	focusedwindow = nil
	require "HBD_fileManager.class"
	require "HBD_fileManager.window"
	require "HBD_fileManager.button"
	lume = require "HBD_fileManager.lume"
	
	windows = {}
	buttons = {}
	
	-- Recording stuff
	love.filesystem.mkdir( "records" )
	recordingdata = {}
	isRecording = false
	timerecorded = 0 --Just so users can see how long they've been recording
	framesrecorded = 0 --Just so users can see how long they've been recording
	recordingtimer = 0
	
	recordingmaxtimer = .025
end

function love.update(dt)
	if hltimer > 0 then
		hltimer = hltimer - dt
		if hltimer < 0 then
			hltimer = 0
		end
	end
	
	for i, v in pairs(windows) do
		v:update(dt)
		if v.delete then
			windows[i] = nil
		end
	end
	
	for i, v in pairs(buttons) do
		v:update(dt)
		if v.delete then
			buttons[i] = nil
		end
	end
	
	if isRecording then
		if recordingtimer >= recordingmaxtimer then
			local a = love.graphics.newScreenshot()
			recordingdata[#recordingdata+1] = a
			framesrecorded = framesrecorded + dt
			timerecorded = round(framesrecorded)
			recordingtimer = 0
		else
			recordingtimer = recordingtimer + dt
		end
	end
	
	require("HBD_fileManager.lurker").update()
end

function love.draw()
	for i, v in pairs(files) do
		local posx = grid[filesimages[i].y][filesimages[i].x].x
		local posy = grid[filesimages[i].y][filesimages[i].x].y
		love.graphics.setColor(255, 255, 255, 255)
		if highlighted then
			if highlighted[1] == filesimages[i].y and highlighted[2] == filesimages[i].x then
				love.graphics.setColor(155, 175, 255, 255)
			end
		end
		love.graphics.draw(filesimages[i].image, posx, posy-scroll, 0, filesscale, filesscale)
		love.graphics.setColor(0, 0, 0, 255)
		local txtposx = posx+16
		love.graphics.printf(filesimages[i].text, txtposx, posy+54-scroll, 30, "center")
	end
	
	for i, v in pairs(windows) do
		love.graphics.setColor(255, 255, 255, 255)
		v:draw()
	end
	
	for i, v in pairs(buttons) do
		love.graphics.setColor(255, 255, 255, 255)
		v:draw()
	end
	
	if consoleenabled then
		if #consoletexts then
			for i, v in pairs(consoletexts) do
				local posy = (i-1)*15+(screenheight-15*(totalcoms+1))
				love.graphics.setColor(0, 0, 0, 155)
				love.graphics.rectangle("fill", 15, posy, 400, 15)
				love.graphics.setColor(255, 255, 255, 255)
				love.graphics.print(v, 15+3, posy+1)
			end
		end
	end
	
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(mouseimg, love.mouse.getX()-16, love.mouse.getY()-16, 0, 1, 1)
end

function love.keypressed(key, unicode)
	if key == "end" or key == "escape" then
		love.event.quit()
	end

	if key == "f3" then
		
		-- PART 1: Get the screenshot itself and make it as an image
		
		local number = 1
		local lnumber = "0001"

		if not love.filesystem.exists("screenshots") then --make one
			love.filesystem.mkdir("screenshots")
		end

		local screenshot = love.graphics.newScreenshot()
		while love.filesystem.exists("screenshots/screenshot_" .. lnumber .. ".png") do
			number = number + 1
			if number >= 1 and number < 10 then
				lnumber = "000" .. tostring(number)
			elseif number >= 10 and number < 100 then
				lnumber = "00" .. tostring(number)
			elseif number >= 100 and number < 1000 then
				lnumber = "0" .. tostring(number)
			else
				lnumber = tostring(number)
			end
		end
		screenshot:encode("screenshots/screenshot_" .. lnumber .. ".png")
		
		
		--PART 2: Remove transparecy from screenshots. I noticed that they make transparent pixels
		
		local image = love.image.newImageData("screenshots/screenshot_" .. lnumber .. ".png")
		local imagewidth = image:getWidth()
		local imageheight = image:getHeight()
		for y = 0, imageheight-1 do
			for x = 0, imagewidth-1 do
				local r, g, b, a = image:getPixel( x, y )
				if a < 255 then
					a = 255
				end
				image:setPixel( x, y, r, g, b, a)
			end
		end
		image:encode("screenshots/screenshot_" .. lnumber .. ".png")
	end
	
	if key == "f5" then
		updatefiles()
	end
	
	if key == "f9" then
		if isRecording then
			stoprecording()
		else
			startrecording()
		end
	end
	
	if key == "f10" then
		if isRecording then
			stoprecording(true)
		end
	end
	
	if key == "f11" then
		if isRecording then
			isRecording = false
			recordingdata = {}
			framesrecorded = 0
			timerecorded = 0
		end
	end
	
	if key == "`" then
		consoleenabled = not consoleenabled
	end
	
	for i, v in pairs(windows) do
		v:keypressed(key, unicode)
	end
	
	for i, v in pairs(buttons) do
		v:keypressed(key, unicode)
	end
	
	if consoleenabled and love.keyboard.isDown("insert") then
		print("key = \"" .. key .. "\"")
	end
end

function love.keyreleased(key, unicode)
	for i, v in pairs(windows) do
		v:keyreleased(key, unicode)
	end
	
	for i, v in pairs(buttons) do
		v:keyreleased(key, unicode)
	end
end

function love.mousepressed(x, y, button)
	if not focusedwindow and not frozenclick then
		if button == "wu" then
			if scroll > 0 then
				scroll = scroll - scrollfactor
			end
		elseif button == "wd" then
			scroll = scroll + scrollfactor
		end
	end
	
	for i, v in pairs(windows) do
		if not frozenclick then
			v:mousepressed(x, y, button)
		end
	end
	
	for i, v in pairs(buttons) do
		if not frozenclick then
			v:mousepressed(x, y, button)
		end
	end
	
	if not focusedwindow and button == "l" and not frozenclick then
		getfileclick(x, y)
	end
	
	frozenclick = false
end

function love.mousereleased(x, y, button)
	for i, v in pairs(windows) do
		v:mousereleased(x, y, button)
	end
	
	for i, v in pairs(buttons) do
		v:mousereleased(x, y, button)
	end
end

function recursiveEnumerate(folder) --by LÖVE, with some adjustments from me
    local lfs = love.filesystem
    local filesTable = lfs.enumerate(folder)
	local returning = {}
    for i,v in ipairs(filesTable) do
        local file = folder.."/"..v
        if lfs.isFile(file) then
            table.insert(returning, string.sub(file, 2, -1)) -- FILE
        elseif lfs.isDirectory(file) then
            returning[v] = recursiveEnumerate(file, fileTree) -- DIRECTORY
        end
    end
    return returning
end

function string:split(delimiter) --Not by me
	local result = {}
	local from	= 1
	local delim_from, delim_to = string.find( self, delimiter, from	 )
	while delim_from do
		table.insert( result, string.sub( self, from , delim_from-1 ) )
		from = delim_to + 1
		delim_from, delim_to = string.find( self, delimiter, from  )
	end
	table.insert( result, string.sub( self, from  ) )
	return result
end

function switch(t, char, replacer) -- By me!
	local returner = t
	local chartable = {}
	for i = 1, string.len(t) do
		if string.sub(t, i, i+string.len(char)-1) == char then
			table.insert(chartable, i)
		end
	end
	
	if not #chartable or #chartable == 0 then
		return returner
	end
	
	for i = 1, #chartable, -1 do
		if i == 1 then
			if chartable[i] == 1 then
				returner = string.sub(returner, string.len(char), -1)
			end
		elseif i == #chartable then
			if chartable[i] == string.len(returner) - string.len(char) then
				returner = string.sub(returner, 1, -string.len(char))
			end
		else
			returner = string.sub(returner, 1, i-1) .. replacer .. string.sub(returner, i+string.len(char), -1)
		end
	end
	
	return returner
end

function table.contains(t, item)
	for a, b in pairs(t) do
		if b == item then
			return true
		end
	end
	return false
end

function startrecording() --By me!
	recordingdata = {}
	isRecording = true
end

function stoprecording(transparency) -- Got this from my Mari0 +Portal mod, but it's mine :)
	local transparency = transparency or false
	isRecording = false
	recordingtimer = 0
	if not love.filesystem.exists( "records" ) then
		love.filesystem.mkdir( "records" )
	end
	local a = "0001"
	local n = 1
	if love.filesystem.exists("records/record_" .. a) then
		while love.filesystem.exists("records/record_" .. a) do
			local zeros = ""
			n = n + 1
			if n >= 1 and n < 10 then
				zeros = "000"
			elseif n >= 10 and n < 100 then
				zeros = "00"
			elseif n >= 100 and n < 1000 then
				zeros = "0"
			end
			a = zeros .. tostring(n)
		end
	end
	n = 0
	love.filesystem.mkdir("records/record_" .. a)
	
	------------------
	
	for i = 1, #recordingdata do
		print("Recording saving at " .. tostring(round(100/#recordingdata*i, 2)) .. "%")
		local b = "0001"
		if love.filesystem.exists("records/record_" .. a .. "/" .. b .. ".png") then
			while love.filesystem.exists("records/record_" .. a .. "/" .. b .. ".png") do
				local zeros = ""
				n = n + 1
				if n >= 1 and n < 10 then
					zeros = "000"
				elseif n >= 10 and n < 100 then
					zeros = "00"
				elseif n >= 100 and n < 1000 then
					zeros = "0"
				end
				b = zeros .. tostring(n)
			end
		end
		
		if transparency then
			for x = 0, recordingdata[i]:getWidth()-1 do
				for y = 0, recordingdata[i]:getHeight()-1 do
					local r, g, b, a = recordingdata[i]:getPixel(x, y)
					a = 255
					recordingdata[i]:setPixel(x, y, r, g, b, a)
				end
			end
		end
		recordingdata[i]:encode("records/record_" .. a .. "/" .. b .. ".png")
	end
	
	recordingdata = {}
	
	framesrecorded = 0
	timerecorded = 0
end

function updatefiles()
	files = recursiveEnumerate("")
	filespos = {}
	filesimages = {}
	totalfiles = 0
	highlighted = {}
	hltimer = 0
	for i, v in pairs(files) do
		totalfiles = totalfiles + 1
		filespos[totalfiles] = i
		if type(v) == "string" then
			print("Files["..i.."] = " .. tostring(v))
		else
			print("Files["..i.."] amount of files = " .. tostring(#v))
		end
		local posy = math.ceil(totalfiles/10)
		local posx = math.mod(totalfiles+10, 10)
		local img = images.docs.empty
		if posx == 0 then posx = 10 end
		local txt = v
		if type(v) == "table" then -- It's a folder
			local istext, isimage, isvideo, ismusic, isfile = 0, 0, 0, 0, false
			img = images.folders.empty
			for j, w in pairs(v) do
				if type(w) == "string" then -- It has files
					isfile = true
					local extension = string.sub(w, -4, -1)
					if table.contains(ext.text, extension) then
						istext = 1
						img = images.folders.doc.text
					elseif table.contains(ext.image, extension) then
						isimage = 1
						img = images.folders.doc.image
					elseif table.contains(ext.video, extension) then
						isvideo = 1
						img = images.folders.doc.video
					elseif table.contains(ext.music, extension) then
						ismusic = 1
						img = images.folders.doc.music
					end
				end
			end
			if isfile then
				if istext+isimage+isvideo+ismusic > 1 then
					img = images.folders.doc.multi
				end
			end
			txt = i
		elseif love.filesystem.isFile(v) then
			local istext, isimage, isvideo, ismusic = 0, 0, 0, 0
			local extension = string.sub(v, -4, -1)
			if table.contains(ext.text, extension) then
				istext = 1
				img = images.docs.text
			elseif table.contains(ext.image, extension) then
				isimage = 1
				img = images.docs.image
			elseif table.contains(ext.video, extension) then
				isvideo = 1
				img = images.docs.video
			elseif table.contains(ext.music, extension) then
				ismusic = 1
				img = images.docs.music
			end
			if istext+isimage+isvideo+ismusic > 1 then
				img = images.docs.multi
			end
			txt = string.sub(v, 1, -5)
		end
		filesimages[i] = {}
		filesimages[i].image = img
		filesimages[i].x = posx
		filesimages[i].y = posy
		filesimages[i].text = txt
	end
	print("Total amount of files = " .. totalfiles)
end

function getfileclick(x, y)
	print("x = " .. x .. ", y = " .. y)
	local posx = math.floor((x - gridoffsets.x)/(griddistances.x))+1
	local posy = math.floor((y - gridoffsets.y)/(griddistances.y))+1
	local generalpos = (posy-1)*10+posx
	print("Click at " .. posx .. "/" .. posy)
	if grid[posy] and grid[posy][posx] then --It has a file where you clicked
		print("Clicked on grid at " .. posx .. "/" .. posy)
		if files[filespos[generalpos]] then
			local newx = grid[posy][posx].x
			local newy = grid[posy][posx].y-scroll
			if x >= newx and x <= newx+64 and y >= newy and y <= newy+64 then -- On correct position
				print("File click on grid")
				if not #highlighted then --Highlight first
					highlighted = {posy, posx}
					hltimer = clicktimer
					print("Highlighted " .. posx .. "/" .. posy)
				elseif highlighted[1] ~= posy or highlighted[2] ~= posx then --Highlight the right file
					highlighted = {posy, posx}
					hltimer = clicktimer
					print("Highlighted " .. posx .. "/" .. posy)
				elseif hltimer == 0 then --Missed the double click time
					hltimer = clicktimer
					print("Missed double click")
				else --Finally, open the window
					print("Open file")
					hltimer = 0
					highlighted = {}
					if type(files[filespos[generalpos]]) == "table" then --It's a folder
						print("File is a folder")
					elseif love.filesystem.isFile(files[filespos[generalpos]]) then --It's a file
						print("File is a file")
						local title = filesimages[filespos[generalpos]].text
						local extension = string.sub(files[filespos[generalpos]], -4, -1)
						if table.contains(ext.text, extension) then
							local data = love.filesystem.read(files[filespos[generalpos]])
							table.insert(windows, window:new(50, 50, 300, 500, "text", title, data))
						elseif table.contains(ext.image, extension) then
							local data = love.graphics.newImage(files[filespos[generalpos]])
							table.insert(windows, window:new(50, 50, data:getWidth(), data:getHeight(), "image", title, data))
						elseif table.contains(ext.video, extension) then
							table.insert(windows, window:new(50, 50, 300, 100, "error", title)) --Because LÖVE can't run videos
						elseif table.contains(ext.music, extension) then
							local data = love.audio.newSource(files[filespos[generalpos]])
							table.insert(windows, window:new(50, 50, 300, 100, "music", title, data))
						end
					end
				end
			else
				highlighted = {}
			end
		else
			highlighted = {}
		end
	else
		highlighted = {}
	end
end

function round(num, idp) --Not by me
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

function print(t)
	rprint(t)
	if #consoletexts == totalcoms then
		table.remove(consoletexts, 1)
	end
	table.insert(consoletexts, t)
end

function getDistance(x1, y1, x2, y2)
	return math.sqrt(math.abs(x1-x2)^2+math.abs(y1-y2)^2)
end